﻿using System;
using System.Configuration;
using System.Globalization;
using System.Text;
using System.Text.RegularExpressions;

namespace RealLifeExample
{
    /// <summary>
    /// Helper class providing string manipulation functionality.
    /// </summary>
    public class StringHelper
    {
        /// <summary>
        /// Default ellipsis for the limit length methods.
        /// </summary>
        public const string DEFAULT_PADDING = "...";

        /// <summary>
        /// Limits the string length.
        /// </summary>
        /// <param name="text">Text to trim</param>
        /// <param name="maxLength">Max length</param>
        /// <param name="cutLocation">Location in the text where the cut of extra text occurs</param>
        public static string LimitLength(string text, int maxLength, LimitedOptions cutLocation)
        {
            return LimitLength(text, maxLength, null, false, cutLocation);
        }

        /// <summary>
        /// Limits the string length.
        /// </summary>
        /// <param name="text">Text to trim</param>
        /// <param name="maxLength">Max length</param>
        /// <param name="padString">Trim character</param>
        /// <param name="wholeWords">If true, the text won't be cut in the middle of the word</param>
        /// <param name="cutLocation">Location in the text where the cut of extra text occurs</param>
        public static string LimitLength(string text, int maxLength, string padding = DEFAULT_PADDING, bool wholeWords = false, LimitedOptions cutLocation = LimitedOptions.End)
        {
            if (text == null)
            {
                return null;
            }

            if (maxLength <= 0)
            {
                return text;
            }

            if (padding == null)
            {
                padding = DEFAULT_PADDING;
            }

            // Trim the text
            int trimmedLength = maxLength - padding.Length;
            int limitTrimmedLength = trimmedLength;
            if (trimmedLength <= 0)
            {
                return padding;
            }

            if (text.Length > maxLength)
            {
                if (cutLocation == LimitedOptions.Middle)
                {
                    // Cut in the middle
                    var halfLength = trimmedLength / 2;

                    return LimitLength(text, halfLength, "", wholeWords) + padding + LimitLength(text, halfLength, "", wholeWords, LimitedOptions.Start);
                }

                bool reversed = (cutLocation == LimitedOptions.Start);

                // Ensure the cut to the whole words
                if (wholeWords && Char.IsLetterOrDigit(GetCharAt(text, trimmedLength, reversed)))
                {
                    // Go to the beginning of the word
                    while ((trimmedLength > 0) && Char.IsLetterOrDigit(GetCharAt(text, trimmedLength - 1, reversed)))
                    {
                        trimmedLength--;
                    }

                    // Go to the end of the next word before (skip the white spaces and special chars)
                    while ((trimmedLength > 0) && !Char.IsLetterOrDigit(GetCharAt(text, trimmedLength - 1, reversed)))
                    {
                        trimmedLength--;
                    }

                    // We don't want to remove whole text if it is one long word
                    if (trimmedLength == 0)
                    {
                        trimmedLength = limitTrimmedLength;
                    }
                }

                if (reversed)
                {
                    text = text.Substring(text.Length - trimmedLength);
                    text = padding + text;
                }
                else
                {
                    text = text.Substring(0, trimmedLength);
                    text = text + padding;
                }
            }

            return text;
        }

        /// <summary>
        /// Gets the character at the given position of the text
        /// </summary>
        /// <param name="text">Source text</param>
        /// <param name="index">Index of the character</param>
        /// <param name="reverse">If true, the character is retrieved in the reversed order (from the end)</param>
        private static char GetCharAt(string text, int index, bool reverse)
        {
            return reverse ? text[text.Length - index - 1] : text[index];
        }


        /// <summary>
        /// Gets the code name created from the given string.
        /// </summary>
        /// <param name="name">Display name</param>
        /// <param name="replacement">Replacement string for invalid characters</param>
        /// <param name="maxLength">Maximal length of the codename</param>
        /// <param name="useUnicode">If true, Unicode letters are allowed in the codename, otherwise only a-Z are allowed</param>
        /// <param name="removeDiacritics">If true, diacritics is removed from Latin characters</param>
        /// <param name="allowedCharacters">Characters which should be allowed within the current name</param>
        /// <param name="useCamelCase">If true, camel case is used for the code name</param>
        public static string GetCodeName(object name, string replacement = null, int maxLength = 0, bool useUnicode = true, bool removeDiacritics = true, string allowedCharacters = null, bool useCamelCase = true)
        {
            string stringName = Convert.ToString(name);
            if (string.IsNullOrEmpty(stringName))
            {
                return "";
            }

            if (replacement == null)
            {
                replacement = "_";
            }

            // Add prefix if defined
            string prefix = ConfigurationManager.AppSettings["CMSCodeNamePrefix"] ?? String.Empty;
            if (!string.IsNullOrEmpty(prefix))
            {
                stringName = prefix + stringName;
            }

            // Remove Latin diacritics if enabled
            if (removeDiacritics)
            {
                stringName = RemoveDiacritics(stringName);
            }

            // Make camel case if required
            if (useCamelCase)
            {
                StringBuilder sb = new StringBuilder();

                // Split to words and join with first letter to upper
                string[] words = stringName.Split(new[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);
                foreach (var w in words)
                {
                    // Make first letter to Upper
                    sb.Append(Char.ToUpperInvariant(w[0]));
                    sb.Append(w.Substring(1));
                }

                stringName = sb.ToString();
            }

            // Replace invalid characters
            stringName = Regex.Replace(stringName, "[^" + (useUnicode ? "\\p{L}" : "a-zA-Z") + "0-9_.-" + allowedCharacters + "]+", replacement);

            // Trim codename length
            if ((maxLength > 0) && (stringName.Length > maxLength))
            {
                stringName = stringName.Substring(0, maxLength);
            }

            // Replace duplicated dots
            stringName = Regex.Replace(stringName, @"\.+", ".");

            // Replace dots at start and end
            stringName = stringName.Trim('.', '_');

            return stringName;
        }

        /// </summary>
        /// <param name="s">Input string</param>
        public static string RemoveDiacritics(string s)
        {
            if (String.IsNullOrEmpty(s))
            {
                return s;
            }

            // Remove invalid characters that prevent normalization
            s = RemoveCharacters(s, UnicodeCategory.OtherNotAssigned);

            s = s.Normalize(NormalizationForm.FormD);

            // Remove diacritics from normalized string
            s = RemoveCharacters(s, UnicodeCategory.NonSpacingMark);

            return s.Normalize(NormalizationForm.FormC);
        }


        /// <summary>
        /// Removes the invalid characters from the given string
        /// </summary>
        /// <param name="s">String to process</param>
        /// <param name="category">Unicode category to remove</param>
        private static string RemoveCharacters(string s, UnicodeCategory category)
        {
            var sb = new StringBuilder();

            foreach (char c in s)
            {
                if (CharUnicodeInfo.GetUnicodeCategory(c) != category)
                {
                    sb.Append(c);
                }
            }

            return sb.ToString();
        }
    }
}
